@@ -20,7 +20,6 @@ from torchvision.datasets import ImageFolder
 OPT = 'adam'  # adam, sgd
 WEIGHT_DECAY = 0.0001
 MOMENTUM = 0.9  # only when OPT is sgd
-BASE_LR = 0.01
 LR_SCHEDULER = 'step'  # step, multistep, reduce_on_plateau
 LR_DECAY_RATE = 0.1
 LR_STEP_SIZE = 5  # only when LR_SCHEDULER is step
@@ -45,14 +44,16 @@ def get_args() -> argparse.Namespace:
     return args
 
 
-def get_optimizer(parameters) -> torch.optim.Optimizer:
+
+
+def get_optimizer(parameters, base_lr) -> torch.optim.Optimizer:
     if OPT == 'adam':
         optimizer = torch.optim.Adam(parameters,
-                                     lr=BASE_LR,
+                                     lr=base_lr,
                                      weight_decay=WEIGHT_DECAY)
     elif OPT == 'sgd':
         optimizer = torch.optim.SGD(parameters,
-                                    lr=BASE_LR,
+                                    lr=base_lr,
                                     weight_decay=WEIGHT_DECAY,
                                     momentum=MOMENTUM)
     else:
@@ -102,20 +103,20 @@ def get_lr_scheduler_config(optimizer: torch.optim.Optimizer) -> dict:
 
 
 class ImageTransform:
-    def __init__(self, is_train: bool, img_size: Union[int, tuple] = 112):
+    def __init__(self, is_train: bool, img_size: Union[int, tuple] = (112,112)):
+        if type(img_size) == int:
+            img_size = (img_size, img_size)
         if is_train:
             self.transform = transforms.Compose([
                 transforms.RandomHorizontalFlip(p=0.5),
-                #transforms.Resize(img_size),
-                transforms.Resize((224,224)),
+                transforms.Resize(img_size),
                 transforms.ToTensor(),
                 transforms.Normalize(mean=[0.485, 0.456, 0.406],
                                      std=[0.229, 0.224, 0.225])
             ])
         else:
             self.transform = transforms.Compose([
-                #transforms.Resize(img_size),
-                transforms.Resize((224,224)),
+                transforms.Resize(img_size),
                 transforms.ToTensor(),
                 transforms.Normalize(mean=[0.485, 0.456, 0.406],
                                      std=[0.229, 0.224, 0.225])
@@ -159,7 +160,7 @@ class SimpleData(LightningDataModule):
 
 class SimpleModel(LightningModule):
     def __init__(self, model_name: str = 'resnet18',
-                 pretrained: bool = False, num_classes: Optional[int] = None, class_to_index=None):
+                 pretrained: bool = False, num_classes: Optional[int] = None, class_to_index=None, img_size=112,base_lr=0.01):
         super().__init__()
         self.save_hyperparameters()
         self.model = timm.create_model(model_name=model_name,
@@ -169,6 +170,7 @@ class SimpleModel(LightningModule):
         self.train_acc = Accuracy()
         self.val_loss = nn.CrossEntropyLoss()
         self.val_acc = Accuracy()
+        self.base_lr = base_lr
 
     def forward(self, x):
         return self.model(x)
@@ -196,7 +198,7 @@ class SimpleModel(LightningModule):
         self.log_dict({'val/loss': loss, 'val/acc': acc})
 
     def configure_optimizers(self):
-        optimizer = get_optimizer(self.parameters())
+        optimizer = get_optimizer(self.parameters(),self.base_lr)
         lr_scheduler_config = get_lr_scheduler_config(optimizer)
         return {"optimizer": optimizer, "lr_scheduler": lr_scheduler_config}
 
@@ -205,20 +207,17 @@ def get_basic_callbacks(checkpoint_interval: int = 1) -> list:
     lr_callback = LearningRateMonitor(logging_interval='epoch')
     ckpt_callback = ModelCheckpoint(filename='epoch{epoch:03d}',
                                     auto_insert_metric_name=False,
-                                    save_top_k=-1,
+                                    save_top_k=1,
+                                    monitor="val/acc",
                                     every_n_epochs=checkpoint_interval)
     return [ckpt_callback, lr_callback]
 
 
-def get_gpu_settings(args: argparse.Namespace) -> tuple:
-    if args.gpu_ids is not None:
+def get_gpu_settings(gpu_ids) -> tuple:
+    if gpu_ids is not None:
         # list
-        gpus = args.gpu_ids
-        strategy = "ddp" if len(gpus) > 1 else None
-    elif args.n_gpu is not None:
-        # int
-        gpus = args.n_gpu
-        strategy = "ddp" if gpus > 1 else None
+        gpus = gpu_ids
+        strategy = "ddp" if len(gpus) > 1 else None   
     else:
         gpus = 1
         strategy = None
@@ -227,33 +226,26 @@ def get_gpu_settings(args: argparse.Namespace) -> tuple:
     return gpus, strategy
 
 
-def get_trainer(args: argparse.Namespace) -> Trainer:
-    callbacks = get_basic_callbacks(checkpoint_interval=args.save_interval)
-    gpus, strategy = get_gpu_settings(args)
+def get_trainer(epochs, outdir, save_interval, gpu_ids) -> Trainer:
+    callbacks = get_basic_callbacks(checkpoint_interval=save_interval)
+    gpus, strategy = get_gpu_settings(gpu_ids)
     trainer = Trainer(
-        max_epochs=args.epochs,
+        max_epochs=epochs,
         callbacks=callbacks,
-        default_root_dir=args.outdir,
+        default_root_dir=outdir,
         gpus=gpus,
         strategy=strategy,
         logger=True,
     )
     return trainer
 
-
-if __name__ == '__main__':
-    args = get_args()
-    seed_everything(args.seed)
-
-    data = SimpleData(root_dir=args.dataset, img_size=args.img_size,
-                      batch_size=args.batch_size, num_workers=args.num_workers)
-    model = SimpleModel(model_name=args.model_name,
+def run_train(dataset_path, model_name="resnet18", outdir="results", img_size=112, epochs=100, save_interval=10, batch_size=8, num_workers=12, gpu_ids=[0], seed=42, base_lr=0.01):
+    seed_everything(seed)
+    data = SimpleData(root_dir=dataset_path, img_size=img_size,
+                      batch_size=batch_size, num_workers=num_workers)
+    model = SimpleModel(model_name=model_name,
                         pretrained=True,
-                        num_classes=len(data.classes), class_to_index=data.class_to_idx)
-    trainer = get_trainer(args)
-
-    print('Args:')
-    pprint(args.__dict__)
-    print('Training classes:')
-    pprint(data.class_to_idx)
+                        num_classes=len(data.classes), class_to_index=data.class_to_idx, img_size=img_size,base_lr=base_lr)
+    trainer = get_trainer(epochs, outdir, save_interval, gpu_ids)
     trainer.fit(model, data)
+
